// Main script for module 'PullDownMenus'

__PullDownMenus PullDownMenus;
export PullDownMenus;

// **********************************************************************
// TYPES
// **********************************************************************

enum EMenuItemType
{
  kMenuItemType_Divider,
  kMenuItemType_Toggle,
  kMenuItemType_Normal
};

#define MAX_MENUS 16
#define MAX_ITEMS 16

struct MenuItem {
  
  String name;
  EMenuItemType type;
  bool enabled;
  bool on;
};

struct Menu {

  String name;
  int numItems;
  String itemNames[MAX_ITEMS];
  String itemRightText[MAX_ITEMS];
  EMenuItemType itemTypes[MAX_ITEMS];
  bool itemEnabled[MAX_ITEMS];
  bool itemOn[MAX_ITEMS];
  int itemIcons[MAX_ITEMS];
  int startX;
  int endX;
};

// **********************************************************************
// GLOBAL DATA (SET BY USER)
// **********************************************************************

int kFont = 0;
int kHSpacing = 10;
int kVSpacing = 2;
int kForegroundColour = 0;
int kBackgroundColour = 1;
int kDisabledColour = 2;
int kBorderColour = 1;
GUI *kMenuBarGUI;
GUI *kMenuItemsGUI;
Character *kCallbackCharacter;
int kCursor = 0;
int kTickIcon = -1;

int kNumMenus = 0;
Menu kMenus[MAX_MENUS];

// **********************************************************************
// GLOBAL DATA (CREATED BY MODULE)
// **********************************************************************

DynamicSprite *kMenuBarSprite;
DynamicSprite *kMenuItemsSprite;
int kMenuBarGraphic = 0;
int kMenuItemsGraphic = 0;
int kOldCursor = 0;

int kTextHeight = 10;

int kCurrentMenu = -1;
int kCurrentItem = -1;
bool kPulledDown = false;

String kClickedMenuName;
String kClickedItemName;

int kFlashTime = 0;
bool kFlashOn = false;
bool kShouldSendCallback = false;

bool kBarDirty = false;

// **********************************************************************
// LOCAL HELPER FUNCTIONS
// **********************************************************************

function TileGraphic(int slot, int w, int h) {

  if (slot != 0)
  {
    // tile the bg graphic
    int x = 0;
    while (x < w)
    {
      int y = 0;
      while (y < h)
      {
        RawDrawImage(x, y, slot);
        y += Game.SpriteHeight[slot];
      }
      x += Game.SpriteWidth[slot];
    }
  }
}

function DrawBorder(int w, int h) {

  if (kBorderColour != kBackgroundColour)
  {
    RawSetColor(kBorderColour);
    RawDrawLine(0, 0, w-1, 0);
    RawDrawLine(w-1, 0, w-1, h-1);
    RawDrawLine(w-1, h-1, 0, h-1);
    RawDrawLine(0, h-1, 0, 0);
  }
}

// **********************************************************************

function CreateMenusSprite() {

  if (kMenuBarGUI != null && kNumMenus > 0)
  {
    RawSaveScreen();
    // get width of sprite
    int w = kMenuBarGUI.Width;
    int h = kMenuBarGUI.Height;
    int textHOff = (h - kTextHeight)/2;

    RawSetColor(kBackgroundColour);
    RawDrawRectangle(0, 0, w, h);
    TileGraphic(kMenuBarGraphic, w, h);
    DrawBorder(w, h);

    int oldFont = Game.NormalFont;
    Game.NormalFont = kFont;
    int m = 0;
    while (m < kNumMenus)
    {
      RawSetColor(kForegroundColour);
      if (m == kCurrentMenu)
      {
        RawDrawRectangle(kMenus[m].startX, 0, kMenus[m].endX, h);
        RawSetColor(kBackgroundColour);
      }
          
      RawPrint(kMenus[m].startX + kHSpacing/2, textHOff, kMenus[m].name);
      m++;
    }
    Game.NormalFont = oldFont;
    
    if (kMenuBarSprite != null)
    {
      kMenuBarSprite.Delete();
    }
    kMenuBarSprite = DynamicSprite.CreateFromBackground(GetBackgroundFrame(), 0, 0, w, h);
    kMenuBarGUI.BackgroundGraphic = 0;
    kMenuBarGUI.BackgroundGraphic = kMenuBarSprite.Graphic;

    RawRestoreScreen();
  }
  
  kBarDirty = false;
}

// **********************************************************************

function CreateItemsSprite() {

  int n = kMenus[kCurrentMenu].numItems;
  if (n > 0)
  {
    int d = kCurrentMenu;
    RawSaveScreen();
    int wmax = 0;
    int i = 0;
    while (i < n)
    {
      if (kMenus[d].itemTypes[i] != kMenuItemType_Divider)
      {
        int w = GetTextWidth(kMenus[d].itemNames[i], kFont);
        if (kMenus[d].itemRightText[i].Length > 0)
        {
          w += kHSpacing + GetTextWidth(kMenus[d].itemRightText[i], kFont);
        }
        if (w > wmax) wmax = w;
      }
      i++;
    }
    wmax += 2*kHSpacing;
    int lineH = kTextHeight + kVSpacing;
    int h = n*lineH;

    RawSetColor(kBackgroundColour);
    RawDrawRectangle(0, 0, wmax, h);
    TileGraphic(kMenuItemsGraphic, wmax, h);
    DrawBorder(wmax, h);

    int oldFont = Game.NormalFont;
    Game.NormalFont = kFont;

    i = 0;
    while (i < n)
    {
      RawSetColor(kForegroundColour);
      int midY = i*lineH + lineH/2;
      int midX = kHSpacing/2;
      if (kMenus[d].itemTypes[i] == kMenuItemType_Divider)
      {
        RawDrawLine(kHSpacing/2, i*lineH + lineH/2, wmax - kHSpacing/2, i*lineH + lineH/2);
      }
      else
      {
        if (i == kCurrentItem && !kFlashOn)
        {
          RawDrawRectangle(0, i*lineH, wmax, (i+1)*lineH);
          RawSetColor(kBackgroundColour);
        }
        if (kMenus[d].itemTypes[i] == kMenuItemType_Toggle && kMenus[d].itemOn[i])
        {
          if (kTickIcon >= 0)
          {            
            int checkW = Game.SpriteWidth[kTickIcon];
            int checkH = Game.SpriteHeight[kTickIcon];
            RawDrawImage(midX - checkW/2, midY - checkH/2, kTickIcon);
          }
          else
          {
            int checkW = GetTextWidth("W", kFont);
            RawDrawLine(midX - checkW/2, midY, midX, midY + checkW/2);
            RawDrawLine(midX, midY + checkW/2, midX + checkW/2, midY - checkW/2);
          }
        }
        else
        if (kMenus[d].itemIcons[i] >= 0)
        {
          int icon = kMenus[d].itemIcons[i];
          int iconW = Game.SpriteWidth[icon];
          int iconH = Game.SpriteHeight[icon];
          RawDrawImage(midX - iconW/2, midY - iconH/2, icon);
        }
        if (!kMenus[d].itemEnabled[i])
        {
          RawSetColor(kDisabledColour);
        }
        RawPrint(kHSpacing, i*lineH + kVSpacing/2, kMenus[d].itemNames[i]);
        if (kMenus[d].itemRightText[i].Length > 0)
        {
          int w = GetTextWidth(kMenus[d].itemRightText[i], kFont);
          RawPrint(wmax - kHSpacing - w, i*lineH + kVSpacing/2, kMenus[d].itemRightText[i]);
        }
      }
      i++;
    }

    Game.NormalFont = oldFont;
    
    if (kMenuItemsSprite != null)
    {
      kMenuItemsSprite.Delete();
    }
    kMenuItemsSprite = DynamicSprite.CreateFromBackground(GetBackgroundFrame(), 0, 0, wmax, h);
    kMenuItemsGUI.SetSize(wmax, h);
    kMenuItemsGUI.BackgroundGraphic = 0;
    kMenuItemsGUI.BackgroundGraphic = kMenuItemsSprite.Graphic;
    kMenuItemsGUI.SetPosition(kMenuBarGUI.X + kMenus[d].startX, kMenuBarGUI.Y + kMenuBarGUI.Height - 1);
    kMenuItemsGUI.Visible = true;
    
    RawRestoreScreen();
  }
  else
  {
    kMenuItemsGUI.Visible = false;
  }
}

// **********************************************************************

int GetMenuIndexByName(String menuName) {

  int m = 0;
  while (m < kNumMenus)
  {
    if (menuName == kMenus[m].name)
    {
      return m;
    }
    m++;
  }
  
  return -1;
}

// **********************************************************************

int GetItemIndexByName(int menu, String itemName) {

  int i = 0;
  while (i < kMenus[menu].numItems)
  {
    if (itemName == kMenus[menu].itemNames[i])
    {
      return i;
    }
    i++;
  }
  return -1;
}

// **********************************************************************

int GetMenuIndexByMousePosition() {

  int ret = -1;
  if (kMenuBarGUI != null && kMenuBarGUI.Visible)
  {
    GUI *theGUI = GUI.GetAtScreenXY(mouse.x, mouse.y);
    
    if (theGUI == kMenuBarGUI)
    {
      int y = mouse.y - kMenuBarGUI.Y;
      if (y >= 0 && y < kMenuBarGUI.Height)
      {
        int x = mouse.x - kMenuBarGUI.X;
        int m = 0;
        while (m < kNumMenus)
        {
          if (x >= kMenus[m].startX && x <= kMenus[m].endX)
          {
            ret = m;
            m = kNumMenus;
          }
          m++;
        }
      }
    }
  }
  return ret;
}

// **********************************************************************

int GetItemIndexByMousePosition() {

  int ret = -1;
  if (kMenuItemsGUI != null && kMenuItemsGUI.Visible)
  {
    GUI *theGUI = GUI.GetAtScreenXY(mouse.x, mouse.y);
    
    if (theGUI == kMenuItemsGUI)
    {
      if (mouse.x >= kMenuItemsGUI.X && mouse.x < kMenuItemsGUI.X + kMenuItemsGUI.Width
         && mouse.y >= kMenuItemsGUI.Y && mouse.y < kMenuItemsGUI.Y + kMenuItemsGUI.Height)
      {
        ret = (mouse.y - kMenuItemsGUI.Y)/(kTextHeight + kVSpacing);
      }
    }
  }
  return ret;
}

// **********************************************************************

function CloseItemsMenu() {
  
  kMenuItemsGUI.Visible = false;
  kCurrentMenu = GetMenuIndexByMousePosition();
  kBarDirty = true;
  kPulledDown = false;
}

// **********************************************************************
// EXPORTED FUNCTIONS
// **********************************************************************

function __PullDownMenus::Reset() {

  kNumMenus = 0;
  if (kMenuBarSprite)
  {
    kMenuBarSprite.Delete();
    kMenuBarSprite = null;
  }
  if (kMenuBarGUI != null)
  {
    kMenuBarGUI.Visible = false;
  }
  kMenuBarGraphic = 0;
  if (kMenuItemsSprite)
  {
    kMenuItemsSprite.Delete();
    kMenuItemsSprite = null;
  }
  if (kMenuItemsGUI != null)
  {
    kMenuItemsGUI.Visible = false;
  }
  kMenuItemsGraphic = 0;
}

// **********************************************************************

int __PullDownMenus::MenuCount() {

  return kNumMenus;
}

// **********************************************************************

function __PullDownMenus::HideMenus() {

  if (kMenuBarGUI != null && kMenuBarGUI.Visible)
  {
    kMenuBarGUI.Visible = false;
  }
  if (kMenuItemsGUI != null && kMenuItemsGUI.Visible)
  {
    kMenuItemsGUI.Visible = false;
  }
  kCurrentMenu = -1;
  kCurrentItem = -1;
}

// **********************************************************************

function __PullDownMenus::ShowMenus() {

  if (kMenuBarGUI != null && !kMenuBarGUI.Visible)
  {
    kMenuBarGUI.Visible = true;
  }
}

// **********************************************************************

function __PullDownMenus::SetBarAndItemsGUIs(GUI *bar, GUI *items) {

  kMenuBarGUI = bar;
  if (kMenuBarGUI != null)
  {
    kMenuBarGUI.Clickable = true;
    kMenuBarGUI.SetPosition(0, 0);
    kMenuBarGUI.Visible = true;
    kMenuBarGraphic = kMenuBarGUI.BackgroundGraphic;
    kBarDirty = true;
  }
  kMenuItemsGUI = items;
  if (kMenuItemsGUI != null)
  {
    kMenuItemsGUI.Clickable = true;
    kMenuItemsGUI.Visible = false;
    kMenuItemsGraphic = kMenuItemsGUI.BackgroundGraphic;
  }
  kOldCursor = Mouse.Mode;
  
}

// **********************************************************************

function __PullDownMenus::SetFontAndSpacing(int font, int hspacing, int vspacing) {

  kFont = font;
  int textWidth = GetTextWidth("Height", font);
  kTextHeight = GetTextHeight("Height", font, textWidth + 1);
  kHSpacing = hspacing;
  kVSpacing = vspacing;
  kBarDirty = true;
}

// **********************************************************************

function __PullDownMenus::SetColours(int backgroundColour, int foregroundColour, int disabledColour, int borderColour) {

  kBackgroundColour = backgroundColour;
  kForegroundColour = foregroundColour;
  kDisabledColour = disabledColour;
  kBorderColour = borderColour;
  kBarDirty = true;
}

// **********************************************************************

function __PullDownMenus::SetCallbackCharacter(Character *callbackCharacter) {

  kCallbackCharacter = callbackCharacter;
}

// **********************************************************************

function __PullDownMenus::SetCursor(int cursor) {

  kCursor = cursor;
}

// **********************************************************************

function __PullDownMenus::SetTickIcon(int tick) {

  kTickIcon = tick;
}

// **********************************************************************

function __PullDownMenus::SetBackgroundGraphics(int barGraphic, int itemsGraphic) {

  kMenuBarGraphic = barGraphic;
  kMenuItemsGraphic = itemsGraphic;
  kBarDirty = true;
}

// **********************************************************************

String __PullDownMenus::GetClickedMenuName() {

  return kClickedMenuName;
}

// **********************************************************************

String __PullDownMenus::GetClickedItemName() {

  return kClickedItemName;
}

// **********************************************************************

function __PullDownMenus::AddMenu(String name) {

  if (kNumMenus < MAX_MENUS)
  {
    int m = kNumMenus;
    kMenus[m].name = name;
    kMenus[m].numItems = 0;
    int textWidth = GetTextWidth(name, kFont);
    if (m == 0)
    {
      kMenus[m].startX = 0;
    }
    else
    {
      kMenus[m].startX = kMenus[m-1].endX + 1;
    }
    kMenus[m].endX = kMenus[m].startX + textWidth + kHSpacing - 1;
    kNumMenus++;
  }
  
  // now generate graphic
  kBarDirty = true;
}

// **********************************************************************

function __PullDownMenus::AddMenuItem(String menuName, String itemName, int icon) {

  // first find the menu by name
  int m = GetMenuIndexByName(menuName);
  if (m != -1)
  {
    // add item to this menu
    int i = kMenus[m].numItems;
    if (i < MAX_ITEMS)
    {
      kMenus[m].itemNames[i] = itemName;
      kMenus[m].itemRightText[i] = "";
      kMenus[m].itemTypes[i] = kMenuItemType_Normal;
      kMenus[m].itemEnabled[i] = true;
      kMenus[m].itemOn[i] = true;
      kMenus[m].itemIcons[i] = icon;
      kMenus[m].numItems++;
    }
  }
}

// **********************************************************************

function __PullDownMenus::AddMenuItemToggle(String menuName, String itemName, bool on) {

  int m = GetMenuIndexByName(menuName);
  if (m != -1)
  {
    int i = kMenus[m].numItems;
    if (i < MAX_ITEMS)
    {
      kMenus[m].itemNames[i] = itemName;
      kMenus[m].itemRightText[i] = "";
      kMenus[m].itemTypes[i] = kMenuItemType_Toggle;
      kMenus[m].itemEnabled[i] = true;
      kMenus[m].itemOn[i] = on;
      kMenus[m].itemIcons[i] = -1;
      kMenus[m].numItems++;
    }
  }
}

// **********************************************************************

function __PullDownMenus::AddMenuItemDivider(String menuName) {

  int m = GetMenuIndexByName(menuName);
  if (m != -1)
  {
    int i = kMenus[m].numItems;
    if (i < MAX_ITEMS)
    {
      kMenus[m].itemNames[i] = "";
      kMenus[m].itemRightText[i] = "";
      kMenus[m].itemTypes[i] = kMenuItemType_Divider;
      kMenus[m].itemEnabled[i] = false;
      kMenus[m].itemOn[i] = true;
      kMenus[m].itemIcons[i] = -1;
      kMenus[m].numItems++;
    }
  }
}

// **********************************************************************

function __PullDownMenus::SetMenuItemRightText(String menuName, String itemName, String rightText) {

  int m = GetMenuIndexByName(menuName);
  if (m != -1)
  {
    int i = GetItemIndexByName(m, itemName);
    if (i != -1)
    {
      kMenus[m].itemRightText[i] = rightText;
    }
  }
}

// **********************************************************************

function __PullDownMenus::EnableMenuItem(String menuName, String itemName) {

  int m = GetMenuIndexByName(menuName);
  if (m != -1)
  {
    int i = GetItemIndexByName(m, itemName);
    if (i != -1)
    {
      kMenus[m].itemEnabled[i] = true;
    }
  }
}

// **********************************************************************

function __PullDownMenus::DisableMenuItem(String menuName, String itemName) {

  int m = GetMenuIndexByName(menuName);
  if (m != -1)
  {
    int i = GetItemIndexByName(m, itemName);
    if (i != -1)
    {
      kMenus[m].itemEnabled[i] = false;
    }
  }
}

// **********************************************************************

function __PullDownMenus::ToggleMenuItem(String menuName, String itemName) {

  int m = GetMenuIndexByName(menuName);
  if (m != -1)
  {
    int i = GetItemIndexByName(m, itemName);
    if (i != -1)
    {
      kMenus[m].itemOn[i] = !kMenus[m].itemOn[i];
    }
  }
}

// **********************************************************************

function __PullDownMenus::ToggleOnMenuItem(String menuName, String itemName) {

  int m = GetMenuIndexByName(menuName);
  if (m != -1)
  {
    int i = GetItemIndexByName(m, itemName);
    if (i != -1)
    {
      kMenus[m].itemOn[i] = true;
    }
  }
}

// **********************************************************************

function __PullDownMenus::ToggleOffMenuItem(String menuName, String itemName) {

  int m = GetMenuIndexByName(menuName);
  if (m != -1)
  {
    int i = GetItemIndexByName(m, itemName);
    if (i != -1)
    {
      kMenus[m].itemOn[i] = false;
    }
  }
}

// **********************************************************************

bool __PullDownMenus::IsMenuItemOn(String menuName, String itemName) {

  int m = GetMenuIndexByName(menuName);
  if (m != -1)
  {
    int i = GetItemIndexByName(m, itemName);
    if (i != -1)
    {
      return kMenus[m].itemOn[i];
    }
  }
  
  return false;
}

// **********************************************************************
// FUNCTIONS CALLED BY AGS
// **********************************************************************

function repeatedly_execute() {

  // having this in repeatedly execute allows the interaction to block
  if (kShouldSendCallback)
  {
    kShouldSendCallback = false;
    if (kCallbackCharacter != null)
    {
      kCallbackCharacter.RunInteraction(eModeInteract);
    }
  }
}

// **********************************************************************

function repeatedly_execute_always() {

  if (kMenuBarGUI != null && kMenuItemsGUI != null)
  {
    if (kFlashTime > 0)
    {
      // update flashing the selected menu item
      kFlashTime--;
      
      if (kFlashTime > 0)
      {
        bool flashOn = true;
        if ((kFlashTime % 8) >= 4) flashOn = false;
        if (flashOn != kFlashOn)
        {
          kFlashOn = flashOn;
          CreateItemsSprite();
        }
      }
      else
      {
        kShouldSendCallback = true;
        CloseItemsMenu();
      }        
    }
    else
    {
      kFlashOn = false;

      // update highlighting the hovered over menu/item
      int m = GetMenuIndexByMousePosition();
      if (m != kCurrentMenu)
      {
        if (m != -1 || !kPulledDown)
        {
          kCurrentMenu = m;
          kBarDirty = true;
          if (m != -1 && kPulledDown)
          {
            CreateItemsSprite();
          }
        }
      }
      int i = GetItemIndexByMousePosition();
      if (i != kCurrentItem)
      {
        kCurrentItem = i;
        if (kCurrentMenu != -1 && kPulledDown)
        {
          CreateItemsSprite();
        }
      }
    }
    
    if (kBarDirty)
    {
      CreateMenusSprite();
    }

    // update the mouse cursor
    if (Mouse.Mode != kCursor)
    {
      kOldCursor = Mouse.Mode;
    }

    GUI *theGUI = GUI.GetAtScreenXY(mouse.x,  mouse.y);

    if (theGUI != kMenuBarGUI && theGUI != kMenuItemsGUI)
    {
      // restore cursor
      if (Mouse.Mode != kOldCursor)
      {
        Mouse.Mode = kOldCursor;
      }
    }
    else
    {
      // set cursor
      if (Mouse.Mode != kCursor)
      {
        Mouse.Mode = kCursor;
      }
    }
  }
}

// **********************************************************************

function on_event(EventType event, int data) {

  if (kMenuBarGUI != null && kMenuItemsGUI != null && kFlashTime == 0)
  {
    if (event == eEventGUIMouseDown)
    {
      int m = GetMenuIndexByMousePosition();
      if (m != -1)
      {
        if (kMenus[m].numItems == 0)
        {
          kMenuItemsGUI.Visible = false;
        }
        else
        {
          kCurrentMenu = m;
          CreateItemsSprite();
          kMenuItemsGUI.Visible = true;
          kCurrentItem = -1;
        }
        kBarDirty = true;
        
        kPulledDown = true;

        ClaimEvent();
      }
    }
    else
    if (event == eEventGUIMouseUp)
    {
      int i = GetItemIndexByMousePosition();
      if (i != -1)
      {
        int d = kCurrentMenu;
        if (kMenus[d].itemTypes[i] == kMenuItemType_Toggle)
        {
          kMenus[d].itemOn[i] = !kMenus[d].itemOn[i];
        }
        if (kMenus[d].itemTypes[i] != kMenuItemType_Divider && kMenus[d].itemEnabled[i])
        {
          // call back to menu d / item i
          kClickedMenuName = kMenus[d].name;
          kClickedItemName = kMenus[d].itemNames[i];
          kFlashTime = GetGameSpeed()/2;
        }
        else
        {
          CloseItemsMenu();
        }
        ClaimEvent();
      }
      else
      {
        CloseItemsMenu();
      }
    }
  }
}

// **********************************************************************


